#!/usr/bin/env python2
# Reproduction recipe for NV-7983 "Infinite Retries"
#
# Try to commit using the FSFS API via Python bindings,
# and test whether first and second commits fail consistently.
#
# Usage: test-commits-5.py REPO_DIR

import sys
from svn import core, repos, fs, delta
from svn.core import SubversionException

try:
    repo_dir = sys.argv[1]
except:
    print("required argument missing: path to repository")
    sys.exit(1)

fs_type = fs.type(repo_dir + '/db')

print("Testing commit failures in " + fs_type + " repo '" + repo_dir + "'")

fs_config = None  # = { "fs-type": fs_type }

repos_pool = core.svn_pool_create()
r = repos.open2(repo_dir, fs_config, repos_pool)
f = repos.fs(r)

# prepare a new txn based on r1 (not on current head)
txn_pool = core.svn_pool_create(repos_pool)
txn = fs.begin_txn(f, 1, txn_pool)
t_name = fs.txn_name(txn)
t_root = fs.txn_root(txn)

# make some change that will fail at commit time
fs.make_file(t_root, "/dir1/file3")

# close the txn by clearing its pool
core.svn_pool_destroy(txn_pool)

# close the fs by clearing the repos pool
core.svn_pool_destroy(repos_pool)

print("Made a change")

repos_pool = core.svn_pool_create()
r = repos.open2(repo_dir, fs_config, repos_pool)
f = repos.fs(r)

# Try making a commit -- twice
for attempt in range(2):
    # try to commit
    # the commit fails in its "rebase to head" phase.
    print("Trying to commit...")

    txn_pool = core.svn_pool_create(repos_pool)
    txn2 = fs.open_txn(f, t_name, txn_pool)
    try:
        fs.commit_txn(txn2)
    except SubversionException as e:
        e_trimmed = e
        while e_trimmed.message == 'traced call':
            e_trimmed = e_trimmed.child
        if e_trimmed.apr_err == 160004 and 'Corrupt node-revision' in e_trimmed.message:
            print("This commit failed due to corruption, as intended:")
            print(e)
        else:
            print("This commit failed unexpectedly:")
            #print(e)
            raise
    else:
        raise Exception("This commit succeeded, but we intended it to fail because of the corruption.")

    core.svn_pool_destroy(txn_pool)

print("TEST PASSED: It appears the commit attempts consistently failed due to the corruption.")

